# coding: utf-8
import hashlib, hmac
try:
    from urllib import urlencode
except:
    from urllib.parse import urlencode
import time

def payload(verb, path, params):
    # url params should be sorted in alphabet
    url_params = urlencode(sorted(params.items()))
    return "{verb}|{path}|{url_params}".format(
        verb = verb.upper(),
        path = path,
        url_params = url_params
    )

class Auth:
    def __init__(self, access_key, secret_key):
        self.access_key = access_key
        self.secret_key = secret_key

    def signed_challenge(self, challenge):
        payload = "%s%s" % (self.access_key, challenge)
        signature = hmac.new(self.secret_key.encode(), payload.encode(), hashlib.sha256).hexdigest()
        return {
            "auth": {
                "access_key": self.access_key,
                "answer": signature
            }
        }

    def signed_params(self, verb, path, params):
        params = params.copy()
        params = self._format_params(params)
        signature = self.sign(verb, path, params)
        params["signature"] = signature
        return params

    def sign(self, verb, path, params):
        return hmac.new(self.secret_key.encode(), payload(verb, path, params).encode(), hashlib.sha256).hexdigest() 

    def _format_params(self, params):
        if not params.get("access_key"):
            params["access_key"] = self.access_key
        if not params.get("tonce"):
            params["tonce"] = int(time.time() * 1000)
        return params



from setuptools import setup

setup(
    name = "ocx_client",
    version = "0.1.0",
    description = "a client library for the bitcoin exchange OCX",
    url = 'https://github.com/ocxapi/ocx-openapi-ruby-client',
    long_description = open("README.md").read(),
    author = "nju520",
    license = "LGPL",
    keywords = "bitcoin ocx ocx.com",
    test_suite = "tests",
    packages = [
        "ocx_client"
    ],
    install_requires = [
        "requests"
    ],
    zip_safe = False
)



# coding: utf-8
import requests
from .auth import Auth
from .error import ClientError

class Client:
    def __init__(
        self,
        endpoint = "https://openapi.ocx.com",
        access_key = None,
        secret_key = None,
        timeout = 60
    ):
        self.endpoint = endpoint
        self.timeout = timeout

        if access_key and secret_key:
            self.access_key = access_key
            self.secret_key = secret_key
            self.auth = Auth(access_key, secret_key)
        else:
            self.auth = False

    def check_auth(self):
        if not self.auth:
            raise ClientError("Missing access key and/or secret key")

    def get_public(self, path, params=None):
        if params is None:
            params = {}
        url = "%s%s" % (self.endpoint, path)

        response = requests.get(url,
            params = params,
            timeout = self.timeout,
            verify = self.endpoint.startswith("https://")
        )

        return self.response_to_dict(response)

    def get(self, path, params=None):
        if params is None:
            params = {}
        self.check_auth()
        url = "%s%s" % (self.endpoint, path)
        params = self.auth.signed_params("get", path, params)

        response = requests.get(url,
            params = params,
            timeout = self.timeout,
            verify = self.endpoint.startswith("https://")
        )

        return self.response_to_dict(response)


    def post(self, path, params=None):
        if params is None:
            params = {}
        self.check_auth()
        url = "%s%s" % (self.endpoint, path)
        params = self.auth.signed_params("post", path, params)

        response = requests.post(url,
            data = params,
            timeout = self.timeout,
            verify = self.endpoint.startswith("https://")
        )

        return self.response_to_dict(response)

    def response_to_dict(self, response):
        try:
            return response.json()
        except ValueError:
            raise ClientError("Response is in bad json format")



# coding: utf-8
from .auth import Auth
from .error import ClientError
import logging
import websocket
import json

class StreamingClient:
    def __init__(self,
        access_key,
        secret_key,
        endpoint = "wss://openapi.ocx.com:8080",
        logger = None
    ):
        self.endpoint = endpoint
        self.access_key = access_key
        self.secret_key = secret_key
        self.auth = Auth(access_key, secret_key)
        if logger is None:
            logger = logging.getLogger(__name__)
            # initialize the logger
            handler = logging.StreamHandler()
            handler.setLevel(logging.INFO)
            logger.setLevel(logging.INFO)
            logger.addHandler(handler)
        self.logger = logger


    def run(self, on_message=None):
        if on_message is not None:
            self.on_message = on_message

        def on_open(ws):
            self.logger.info("Connected!")

        def on_message(ws, message):
            msg = json.loads(message)
            if "challenge" in msg:
                data = msg["challenge"]
                ws.send(json.dumps(self.auth.signed_challenge(data)))
            else:
                try:
                    self.on_message(msg)
                except Exception as e:
                    self.logger.error(e)
                    self.logger.error("Failed to process message: %s" % message)

        def on_close(ws):
            self.logger.info("Closed!")

        def on_error(ws, error):
            self.logger.error(error)
            raise ClientError(error)

        ws = websocket.WebSocketApp(self.endpoint)
        ws.on_open = on_open
        ws.on_message = on_message
        ws.on_close = on_close
        ws.on_error = on_error
        ws.run_forever()
